from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pay_periods import PayPeriods
from ...models.pay_run_entry import PayRunEntry
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    json_body: PayRunEntry,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}/{periodNumber}/{id}".format(
        client.base_url,
        employerId=employer_id,
        taxYear=tax_year,
        payPeriod=pay_period,
        periodNumber=period_number,
        id=id,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[PayRunEntry]:
    if response.status_code == 200:
        response_200 = PayRunEntry.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[PayRunEntry]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    json_body: PayRunEntry,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRunEntry]:
    """Update a PayRunEntry

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (PayRunEntry): A PayRun contains multiple PayRunEntries, one for each Employee
            that is being paid.
            The value for <code>PersonalDetails</code> and <code>EmploymentDetails</code> is copied
            from the Employee record.
            So to change them you should update the Employee, not the PayRunEntry.
            Changes made to the Employee are only reflected whilst the PayRun is still open.
            Once you finalise and close the PayRun then a snapshot is taken of these values which is
            stored with the PayRunEntry for future reference.

    Returns:
        Response[PayRunEntry]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    json_body: PayRunEntry,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRunEntry]:
    """Update a PayRunEntry

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (PayRunEntry): A PayRun contains multiple PayRunEntries, one for each Employee
            that is being paid.
            The value for <code>PersonalDetails</code> and <code>EmploymentDetails</code> is copied
            from the Employee record.
            So to change them you should update the Employee, not the PayRunEntry.
            Changes made to the Employee are only reflected whilst the PayRun is still open.
            Once you finalise and close the PayRun then a snapshot is taken of these values which is
            stored with the PayRunEntry for future reference.

    Returns:
        Response[PayRunEntry]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    json_body: PayRunEntry,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRunEntry]:
    """Update a PayRunEntry

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (PayRunEntry): A PayRun contains multiple PayRunEntries, one for each Employee
            that is being paid.
            The value for <code>PersonalDetails</code> and <code>EmploymentDetails</code> is copied
            from the Employee record.
            So to change them you should update the Employee, not the PayRunEntry.
            Changes made to the Employee are only reflected whilst the PayRun is still open.
            Once you finalise and close the PayRun then a snapshot is taken of these values which is
            stored with the PayRunEntry for future reference.

    Returns:
        Response[PayRunEntry]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        id=id,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    id: str,
    *,
    client: Client,
    json_body: PayRunEntry,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRunEntry]:
    """Update a PayRunEntry

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        id (str):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (PayRunEntry): A PayRun contains multiple PayRunEntries, one for each Employee
            that is being paid.
            The value for <code>PersonalDetails</code> and <code>EmploymentDetails</code> is copied
            from the Employee record.
            So to change them you should update the Employee, not the PayRunEntry.
            Changes made to the Employee are only reflected whilst the PayRun is still open.
            Once you finalise and close the PayRun then a snapshot is taken of these values which is
            stored with the PayRunEntry for future reference.

    Returns:
        Response[PayRunEntry]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            period_number=period_number,
            id=id,
            client=client,
            json_body=json_body,
            ordinal=ordinal,
        )
    ).parsed



from enum import Enum


class EntityType(str, Enum):
    NONE = "None"
    EMPLOYER = "Employer"
    EMPLOYEE = "Employee"
    PAYRUNENTRY = "PayRunEntry"
    PENSIONSCHEME = "PensionScheme"
    PAYCODE = "PayCode"
    NOTE = "Note"
    LEAVE = "Leave"
    BENEFITS = "Benefits"
    PENSION = "Pension"
    ATTACHMENTORDER = "AttachmentOrder"
    OPENINGBALANCES = "OpeningBalances"
    NICSUMMARY = "NicSummary"
    HMRCPAYMENT = "HmrcPayment"
    DPSNOTICE = "DpsNotice"
    USER = "User"
    SETTINGS = "Settings"
    PAYRUN = "PayRun"
    LOAN = "Loan"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pay_code_calculation_type import PayCodeCalculationType
from ..models.pay_code_multiplier_type import PayCodeMultiplierType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayCode")


@attr.s(auto_attribs=True)
class PayCode:
    """Each PayLine has a Code. The Code will match the Code property of a PayCode.
    The PayCode that is used determines how the amount is treated with regards to tax, NI and pensions

        Attributes:
            title (str):
            code (str):
            default_value (Union[Unset, None, float]): If a non-zero value is provided then when this code is selected in
                our UI the value will be automatically filled.
            is_deduction (Union[Unset, bool]): If set to true then a PayLine using this code will be treated as a deduction
                as opposed to an addition.
            is_niable (Union[Unset, bool]): Determines whether or not the value of PayLines using this code should be taken
                into consideration when calculating National Insurance Contributions.
            is_taxable (Union[Unset, bool]): Determines whether or not the value of PayLines using this code should be taken
                into consideration when calculating the PAYE liability.
            is_pensionable (Union[Unset, bool]): Determines whether or not the value of PayLines using this code should be
                taken into consideration when calculating the total pensionable pay.
            is_attachable (Union[Unset, bool]): Determines whether or not the value of PayLines using this code should be
                taken into consideration when calculating the attachable pay for AttachmentOrders.
            is_real_time_class_1_a_niable (Union[Unset, bool]): Determines whether or not the value of PayLines using this
                code are subject to real time Class 1A NIC.
            is_not_contributing_to_holiday_pay (Union[Unset, bool]): If true then any payments made using this code will not
                contribute towards holiday pay calculations.
            is_qualifying_earnings_for_ae (Union[Unset, None, bool]): If true then any payments made using this code will be
                used as the basis for calculating QualifyingEarnings for AutoEnrolment assessments.
                If it is set to null then when it's next retrieved it'll have the same value as PensionablePay
            is_not_tierable (Union[Unset, bool]): If true then any payments made using this code will not be included when
                calculating the tier an employee should be on for a Tiered Pension.
            is_tcp_tcls (Union[Unset, bool]): If true then any payments made using this code will be reported as
                a Trivial Commutation Payment (A - TCLS)
            is_tcp_pp (Union[Unset, bool]): If true then any payments made using this code will be reported as
                a Trivial Commutation Payment (B - Personal Pension)
            is_tcp_op (Union[Unset, bool]): If true then any payments made using this code will be reported as
                a Trivial Commutation Payment (C - Occupational Pension)
            is_flexi_dd_death_benefit (Union[Unset, bool]): If true then any payments made using this code will be reported
                as
                a Flexible Drawdown of Death Benefit
            is_flexi_dd_pension (Union[Unset, bool]): If true then any payments made using this code will be reported as
                a Flexible Drawdown flexibly accessing a pension
            calculation_type (Union[Unset, PayCodeCalculationType]):
            multiplier_type (Union[Unset, PayCodeMultiplierType]):
            daily_rate_multiplier (Union[Unset, float]): If the MultiplierType is MultipleOfDailyRate then this sets the
                multiple to be used, ie 1.5
            hourly_rate_multiplier (Union[Unset, float]): If the MultiplierType is MultipleOfHourlyRate then this sets the
                multiple to be used, ie 1.5
            is_system_code (Union[Unset, bool]): [readonly] System Codes cannot be deleted or edited
            is_control_code (Union[Unset, bool]): [readonly] Control Codes cannot be deleted or edited nor can PayLines be
                assigned to them.
            payee (Union[Unset, None, str]): The Id of the Payee, if any, that deductions are to be paid to.
    """

    title: str
    code: str
    default_value: Union[Unset, None, float] = UNSET
    is_deduction: Union[Unset, bool] = UNSET
    is_niable: Union[Unset, bool] = UNSET
    is_taxable: Union[Unset, bool] = UNSET
    is_pensionable: Union[Unset, bool] = UNSET
    is_attachable: Union[Unset, bool] = UNSET
    is_real_time_class_1_a_niable: Union[Unset, bool] = UNSET
    is_not_contributing_to_holiday_pay: Union[Unset, bool] = UNSET
    is_qualifying_earnings_for_ae: Union[Unset, None, bool] = UNSET
    is_not_tierable: Union[Unset, bool] = UNSET
    is_tcp_tcls: Union[Unset, bool] = UNSET
    is_tcp_pp: Union[Unset, bool] = UNSET
    is_tcp_op: Union[Unset, bool] = UNSET
    is_flexi_dd_death_benefit: Union[Unset, bool] = UNSET
    is_flexi_dd_pension: Union[Unset, bool] = UNSET
    calculation_type: Union[Unset, PayCodeCalculationType] = UNSET
    multiplier_type: Union[Unset, PayCodeMultiplierType] = UNSET
    daily_rate_multiplier: Union[Unset, float] = UNSET
    hourly_rate_multiplier: Union[Unset, float] = UNSET
    is_system_code: Union[Unset, bool] = UNSET
    is_control_code: Union[Unset, bool] = UNSET
    payee: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        code = self.code
        default_value = self.default_value
        is_deduction = self.is_deduction
        is_niable = self.is_niable
        is_taxable = self.is_taxable
        is_pensionable = self.is_pensionable
        is_attachable = self.is_attachable
        is_real_time_class_1_a_niable = self.is_real_time_class_1_a_niable
        is_not_contributing_to_holiday_pay = self.is_not_contributing_to_holiday_pay
        is_qualifying_earnings_for_ae = self.is_qualifying_earnings_for_ae
        is_not_tierable = self.is_not_tierable
        is_tcp_tcls = self.is_tcp_tcls
        is_tcp_pp = self.is_tcp_pp
        is_tcp_op = self.is_tcp_op
        is_flexi_dd_death_benefit = self.is_flexi_dd_death_benefit
        is_flexi_dd_pension = self.is_flexi_dd_pension
        calculation_type: Union[Unset, str] = UNSET
        if not isinstance(self.calculation_type, Unset):
            calculation_type = self.calculation_type.value

        multiplier_type: Union[Unset, str] = UNSET
        if not isinstance(self.multiplier_type, Unset):
            multiplier_type = self.multiplier_type.value

        daily_rate_multiplier = self.daily_rate_multiplier
        hourly_rate_multiplier = self.hourly_rate_multiplier
        is_system_code = self.is_system_code
        is_control_code = self.is_control_code
        payee = self.payee

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "title": title,
                "code": code,
            }
        )
        if default_value is not UNSET:
            field_dict["defaultValue"] = default_value
        if is_deduction is not UNSET:
            field_dict["isDeduction"] = is_deduction
        if is_niable is not UNSET:
            field_dict["isNiable"] = is_niable
        if is_taxable is not UNSET:
            field_dict["isTaxable"] = is_taxable
        if is_pensionable is not UNSET:
            field_dict["isPensionable"] = is_pensionable
        if is_attachable is not UNSET:
            field_dict["isAttachable"] = is_attachable
        if is_real_time_class_1_a_niable is not UNSET:
            field_dict["isRealTimeClass1aNiable"] = is_real_time_class_1_a_niable
        if is_not_contributing_to_holiday_pay is not UNSET:
            field_dict[
                "isNotContributingToHolidayPay"
            ] = is_not_contributing_to_holiday_pay
        if is_qualifying_earnings_for_ae is not UNSET:
            field_dict["isQualifyingEarningsForAe"] = is_qualifying_earnings_for_ae
        if is_not_tierable is not UNSET:
            field_dict["isNotTierable"] = is_not_tierable
        if is_tcp_tcls is not UNSET:
            field_dict["isTcp_Tcls"] = is_tcp_tcls
        if is_tcp_pp is not UNSET:
            field_dict["isTcp_Pp"] = is_tcp_pp
        if is_tcp_op is not UNSET:
            field_dict["isTcp_Op"] = is_tcp_op
        if is_flexi_dd_death_benefit is not UNSET:
            field_dict["isFlexiDd_DeathBenefit"] = is_flexi_dd_death_benefit
        if is_flexi_dd_pension is not UNSET:
            field_dict["isFlexiDd_Pension"] = is_flexi_dd_pension
        if calculation_type is not UNSET:
            field_dict["calculationType"] = calculation_type
        if multiplier_type is not UNSET:
            field_dict["multiplierType"] = multiplier_type
        if daily_rate_multiplier is not UNSET:
            field_dict["dailyRateMultiplier"] = daily_rate_multiplier
        if hourly_rate_multiplier is not UNSET:
            field_dict["hourlyRateMultiplier"] = hourly_rate_multiplier
        if is_system_code is not UNSET:
            field_dict["isSystemCode"] = is_system_code
        if is_control_code is not UNSET:
            field_dict["isControlCode"] = is_control_code
        if payee is not UNSET:
            field_dict["payee"] = payee

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        code = d.pop("code")

        default_value = d.pop("defaultValue", UNSET)

        is_deduction = d.pop("isDeduction", UNSET)

        is_niable = d.pop("isNiable", UNSET)

        is_taxable = d.pop("isTaxable", UNSET)

        is_pensionable = d.pop("isPensionable", UNSET)

        is_attachable = d.pop("isAttachable", UNSET)

        is_real_time_class_1_a_niable = d.pop("isRealTimeClass1aNiable", UNSET)

        is_not_contributing_to_holiday_pay = d.pop(
            "isNotContributingToHolidayPay", UNSET
        )

        is_qualifying_earnings_for_ae = d.pop("isQualifyingEarningsForAe", UNSET)

        is_not_tierable = d.pop("isNotTierable", UNSET)

        is_tcp_tcls = d.pop("isTcp_Tcls", UNSET)

        is_tcp_pp = d.pop("isTcp_Pp", UNSET)

        is_tcp_op = d.pop("isTcp_Op", UNSET)

        is_flexi_dd_death_benefit = d.pop("isFlexiDd_DeathBenefit", UNSET)

        is_flexi_dd_pension = d.pop("isFlexiDd_Pension", UNSET)

        _calculation_type = d.pop("calculationType", UNSET)
        calculation_type: Union[Unset, PayCodeCalculationType]
        if isinstance(_calculation_type, Unset):
            calculation_type = UNSET
        else:
            calculation_type = PayCodeCalculationType(_calculation_type)

        _multiplier_type = d.pop("multiplierType", UNSET)
        multiplier_type: Union[Unset, PayCodeMultiplierType]
        if isinstance(_multiplier_type, Unset):
            multiplier_type = UNSET
        else:
            multiplier_type = PayCodeMultiplierType(_multiplier_type)

        daily_rate_multiplier = d.pop("dailyRateMultiplier", UNSET)

        hourly_rate_multiplier = d.pop("hourlyRateMultiplier", UNSET)

        is_system_code = d.pop("isSystemCode", UNSET)

        is_control_code = d.pop("isControlCode", UNSET)

        payee = d.pop("payee", UNSET)

        pay_code = cls(
            title=title,
            code=code,
            default_value=default_value,
            is_deduction=is_deduction,
            is_niable=is_niable,
            is_taxable=is_taxable,
            is_pensionable=is_pensionable,
            is_attachable=is_attachable,
            is_real_time_class_1_a_niable=is_real_time_class_1_a_niable,
            is_not_contributing_to_holiday_pay=is_not_contributing_to_holiday_pay,
            is_qualifying_earnings_for_ae=is_qualifying_earnings_for_ae,
            is_not_tierable=is_not_tierable,
            is_tcp_tcls=is_tcp_tcls,
            is_tcp_pp=is_tcp_pp,
            is_tcp_op=is_tcp_op,
            is_flexi_dd_death_benefit=is_flexi_dd_death_benefit,
            is_flexi_dd_pension=is_flexi_dd_pension,
            calculation_type=calculation_type,
            multiplier_type=multiplier_type,
            daily_rate_multiplier=daily_rate_multiplier,
            hourly_rate_multiplier=hourly_rate_multiplier,
            is_system_code=is_system_code,
            is_control_code=is_control_code,
            payee=payee,
        )

        return pay_code



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.calendar_entry_type import CalendarEntryType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CalendarEntry")


@attr.s(auto_attribs=True)
class CalendarEntry:
    """
    Attributes:
        title (Union[Unset, None, str]):
        start (Union[Unset, datetime.date]):
        end (Union[Unset, datetime.date]):
        type (Union[Unset, CalendarEntryType]):
    """

    title: Union[Unset, None, str] = UNSET
    start: Union[Unset, datetime.date] = UNSET
    end: Union[Unset, datetime.date] = UNSET
    type: Union[Unset, CalendarEntryType] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.date]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start).date()

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.date]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end).date()

        _type = d.pop("type", UNSET)
        type: Union[Unset, CalendarEntryType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CalendarEntryType(_type)

        calendar_entry = cls(
            title=title,
            start=start,
            end=end,
            type=type,
        )

        return calendar_entry



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...models.nic_summary import NicSummary
from ...models.tax_year import TaxYear
from ...types import Response


def _get_kwargs(
    employer_id: str,
    employee_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: List[NicSummary],
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{employeeId}/openingBalances/nic/{taxYear}".format(
        client.base_url,
        employerId=employer_id,
        employeeId=employee_id,
        taxYear=tax_year,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = []
    for json_body_item_data in json_body:
        json_body_item = json_body_item_data.to_dict()

        json_json_body.append(json_body_item)

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, List[NicSummary]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = NicSummary.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, List[NicSummary]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    employee_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: List[NicSummary],
) -> Response[Union[Any, List[NicSummary]]]:
    """Update NicSummaries

     Updates the NicSummaries for an Employee for a given TaxYear.
    If the TaxYear is the same as on their OpeningBalances then the NicSummaries there will also be
    updated.

    Args:
        employer_id (str):
        employee_id (str):
        tax_year (TaxYear):
        json_body (List[NicSummary]):

    Returns:
        Response[Union[Any, List[NicSummary]]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    employee_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: List[NicSummary],
) -> Optional[Union[Any, List[NicSummary]]]:
    """Update NicSummaries

     Updates the NicSummaries for an Employee for a given TaxYear.
    If the TaxYear is the same as on their OpeningBalances then the NicSummaries there will also be
    updated.

    Args:
        employer_id (str):
        employee_id (str):
        tax_year (TaxYear):
        json_body (List[NicSummary]):

    Returns:
        Response[Union[Any, List[NicSummary]]]
    """

    return sync_detailed(
        employer_id=employer_id,
        employee_id=employee_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    employee_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: List[NicSummary],
) -> Response[Union[Any, List[NicSummary]]]:
    """Update NicSummaries

     Updates the NicSummaries for an Employee for a given TaxYear.
    If the TaxYear is the same as on their OpeningBalances then the NicSummaries there will also be
    updated.

    Args:
        employer_id (str):
        employee_id (str):
        tax_year (TaxYear):
        json_body (List[NicSummary]):

    Returns:
        Response[Union[Any, List[NicSummary]]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        tax_year=tax_year,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    employee_id: str,
    tax_year: TaxYear,
    *,
    client: Client,
    json_body: List[NicSummary],
) -> Optional[Union[Any, List[NicSummary]]]:
    """Update NicSummaries

     Updates the NicSummaries for an Employee for a given TaxYear.
    If the TaxYear is the same as on their OpeningBalances then the NicSummaries there will also be
    updated.

    Args:
        employer_id (str):
        employee_id (str):
        tax_year (TaxYear):
        json_body (List[NicSummary]):

    Returns:
        Response[Union[Any, List[NicSummary]]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            employee_id=employee_id,
            tax_year=tax_year,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..models.rti_employee_name import RtiEmployeeName
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisStatement")


@attr.s(auto_attribs=True)
class CisStatement:
    """CIS Payment and Deduction Statement (CISOL1)

    Attributes:
        tax_year (Union[Unset, TaxYear]):
        tax_month (Union[Unset, int]): [readonly]
        contractor_name (Union[Unset, None, str]): [readonly]
        contractor_address (Union[Unset, Address]):
        contractor_office_number (Union[Unset, None, str]): [readonly]
        contractor_paye_reference (Union[Unset, None, str]): [readonly]
        subcontractor_name (Union[Unset, RtiEmployeeName]):
        sub_contractor_utr (Union[Unset, None, str]): [readonly]
        verification_number (Union[Unset, None, str]): [readonly]
        gross_amount_paid (Union[Unset, float]): [readonly] Gross Amount Paid (Excl VAT)
        cost_of_materials (Union[Unset, float]): [readonly]
        amount_liable_to_deduction (Union[Unset, float]): [readonly]
        amount_deducted (Union[Unset, float]): [readonly]
        amount_payable (Union[Unset, float]): [readonly]
    """

    tax_year: Union[Unset, TaxYear] = UNSET
    tax_month: Union[Unset, int] = UNSET
    contractor_name: Union[Unset, None, str] = UNSET
    contractor_address: Union[Unset, Address] = UNSET
    contractor_office_number: Union[Unset, None, str] = UNSET
    contractor_paye_reference: Union[Unset, None, str] = UNSET
    subcontractor_name: Union[Unset, RtiEmployeeName] = UNSET
    sub_contractor_utr: Union[Unset, None, str] = UNSET
    verification_number: Union[Unset, None, str] = UNSET
    gross_amount_paid: Union[Unset, float] = UNSET
    cost_of_materials: Union[Unset, float] = UNSET
    amount_liable_to_deduction: Union[Unset, float] = UNSET
    amount_deducted: Union[Unset, float] = UNSET
    amount_payable: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        tax_month = self.tax_month
        contractor_name = self.contractor_name
        contractor_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contractor_address, Unset):
            contractor_address = self.contractor_address.to_dict()

        contractor_office_number = self.contractor_office_number
        contractor_paye_reference = self.contractor_paye_reference
        subcontractor_name: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.subcontractor_name, Unset):
            subcontractor_name = self.subcontractor_name.to_dict()

        sub_contractor_utr = self.sub_contractor_utr
        verification_number = self.verification_number
        gross_amount_paid = self.gross_amount_paid
        cost_of_materials = self.cost_of_materials
        amount_liable_to_deduction = self.amount_liable_to_deduction
        amount_deducted = self.amount_deducted
        amount_payable = self.amount_payable

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if tax_month is not UNSET:
            field_dict["taxMonth"] = tax_month
        if contractor_name is not UNSET:
            field_dict["contractorName"] = contractor_name
        if contractor_address is not UNSET:
            field_dict["contractorAddress"] = contractor_address
        if contractor_office_number is not UNSET:
            field_dict["contractorOfficeNumber"] = contractor_office_number
        if contractor_paye_reference is not UNSET:
            field_dict["contractorPayeReference"] = contractor_paye_reference
        if subcontractor_name is not UNSET:
            field_dict["subcontractorName"] = subcontractor_name
        if sub_contractor_utr is not UNSET:
            field_dict["subContractorUtr"] = sub_contractor_utr
        if verification_number is not UNSET:
            field_dict["verificationNumber"] = verification_number
        if gross_amount_paid is not UNSET:
            field_dict["grossAmountPaid"] = gross_amount_paid
        if cost_of_materials is not UNSET:
            field_dict["costOfMaterials"] = cost_of_materials
        if amount_liable_to_deduction is not UNSET:
            field_dict["amountLiableToDeduction"] = amount_liable_to_deduction
        if amount_deducted is not UNSET:
            field_dict["amountDeducted"] = amount_deducted
        if amount_payable is not UNSET:
            field_dict["amountPayable"] = amount_payable

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        tax_month = d.pop("taxMonth", UNSET)

        contractor_name = d.pop("contractorName", UNSET)

        _contractor_address = d.pop("contractorAddress", UNSET)
        contractor_address: Union[Unset, Address]
        if isinstance(_contractor_address, Unset):
            contractor_address = UNSET
        else:
            contractor_address = Address.from_dict(_contractor_address)

        contractor_office_number = d.pop("contractorOfficeNumber", UNSET)

        contractor_paye_reference = d.pop("contractorPayeReference", UNSET)

        _subcontractor_name = d.pop("subcontractorName", UNSET)
        subcontractor_name: Union[Unset, RtiEmployeeName]
        if isinstance(_subcontractor_name, Unset):
            subcontractor_name = UNSET
        else:
            subcontractor_name = RtiEmployeeName.from_dict(_subcontractor_name)

        sub_contractor_utr = d.pop("subContractorUtr", UNSET)

        verification_number = d.pop("verificationNumber", UNSET)

        gross_amount_paid = d.pop("grossAmountPaid", UNSET)

        cost_of_materials = d.pop("costOfMaterials", UNSET)

        amount_liable_to_deduction = d.pop("amountLiableToDeduction", UNSET)

        amount_deducted = d.pop("amountDeducted", UNSET)

        amount_payable = d.pop("amountPayable", UNSET)

        cis_statement = cls(
            tax_year=tax_year,
            tax_month=tax_month,
            contractor_name=contractor_name,
            contractor_address=contractor_address,
            contractor_office_number=contractor_office_number,
            contractor_paye_reference=contractor_paye_reference,
            subcontractor_name=subcontractor_name,
            sub_contractor_utr=sub_contractor_utr,
            verification_number=verification_number,
            gross_amount_paid=gross_amount_paid,
            cost_of_materials=cost_of_materials,
            amount_liable_to_deduction=amount_liable_to_deduction,
            amount_deducted=amount_deducted,
            amount_payable=amount_payable,
        )

        return cis_statement



from enum import Enum


class BenefitDetailsAssetType(str, Enum):
    OTHER = "Other"
    MULTIPLE = "Multiple"
    PROPERTY = "Property"
    CARS = "Cars"
    PRECIOUSMETALS = "PreciousMetals"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TaxYear(str, Enum):
    YEAR2017 = "Year2017"
    YEAR2018 = "Year2018"
    YEAR2019 = "Year2019"
    YEAR2020 = "Year2020"
    YEAR2021 = "Year2021"
    YEAR2022 = "Year2022"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="BenefitDetailsLoan")


@attr.s(auto_attribs=True)
class BenefitDetailsLoan:
    """
    Attributes:
        number_of_joint_borrowers (Union[Unset, None, int]):
        loan_made (Union[Unset, None, datetime.date]):
        loan_discharged (Union[Unset, None, datetime.date]):
        starting_balance (Union[Unset, float]):
        closing_balance (Union[Unset, float]):
        max_balance_in_year (Union[Unset, float]):
        interest_paid (Union[Unset, float]):
        cash_equivalent (Union[Unset, float]): [readonly]
        full_tax_months (Union[Unset, int]): [readonly]
        monthly_value (Union[Unset, float]): [readonly]
        official_interest (Union[Unset, float]): [readonly]
    """

    number_of_joint_borrowers: Union[Unset, None, int] = UNSET
    loan_made: Union[Unset, None, datetime.date] = UNSET
    loan_discharged: Union[Unset, None, datetime.date] = UNSET
    starting_balance: Union[Unset, float] = UNSET
    closing_balance: Union[Unset, float] = UNSET
    max_balance_in_year: Union[Unset, float] = UNSET
    interest_paid: Union[Unset, float] = UNSET
    cash_equivalent: Union[Unset, float] = UNSET
    full_tax_months: Union[Unset, int] = UNSET
    monthly_value: Union[Unset, float] = UNSET
    official_interest: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        number_of_joint_borrowers = self.number_of_joint_borrowers
        loan_made: Union[Unset, None, str] = UNSET
        if not isinstance(self.loan_made, Unset):
            loan_made = self.loan_made.isoformat() if self.loan_made else None

        loan_discharged: Union[Unset, None, str] = UNSET
        if not isinstance(self.loan_discharged, Unset):
            loan_discharged = (
                self.loan_discharged.isoformat() if self.loan_discharged else None
            )

        starting_balance = self.starting_balance
        closing_balance = self.closing_balance
        max_balance_in_year = self.max_balance_in_year
        interest_paid = self.interest_paid
        cash_equivalent = self.cash_equivalent
        full_tax_months = self.full_tax_months
        monthly_value = self.monthly_value
        official_interest = self.official_interest

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if number_of_joint_borrowers is not UNSET:
            field_dict["numberOfJointBorrowers"] = number_of_joint_borrowers
        if loan_made is not UNSET:
            field_dict["loanMade"] = loan_made
        if loan_discharged is not UNSET:
            field_dict["loanDischarged"] = loan_discharged
        if starting_balance is not UNSET:
            field_dict["startingBalance"] = starting_balance
        if closing_balance is not UNSET:
            field_dict["closingBalance"] = closing_balance
        if max_balance_in_year is not UNSET:
            field_dict["maxBalanceInYear"] = max_balance_in_year
        if interest_paid is not UNSET:
            field_dict["interestPaid"] = interest_paid
        if cash_equivalent is not UNSET:
            field_dict["cashEquivalent"] = cash_equivalent
        if full_tax_months is not UNSET:
            field_dict["fullTaxMonths"] = full_tax_months
        if monthly_value is not UNSET:
            field_dict["monthlyValue"] = monthly_value
        if official_interest is not UNSET:
            field_dict["officialInterest"] = official_interest

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        number_of_joint_borrowers = d.pop("numberOfJointBorrowers", UNSET)

        _loan_made = d.pop("loanMade", UNSET)
        loan_made: Union[Unset, None, datetime.date]
        if _loan_made is None:
            loan_made = None
        elif isinstance(_loan_made, Unset):
            loan_made = UNSET
        else:
            loan_made = isoparse(_loan_made).date()

        _loan_discharged = d.pop("loanDischarged", UNSET)
        loan_discharged: Union[Unset, None, datetime.date]
        if _loan_discharged is None:
            loan_discharged = None
        elif isinstance(_loan_discharged, Unset):
            loan_discharged = UNSET
        else:
            loan_discharged = isoparse(_loan_discharged).date()

        starting_balance = d.pop("startingBalance", UNSET)

        closing_balance = d.pop("closingBalance", UNSET)

        max_balance_in_year = d.pop("maxBalanceInYear", UNSET)

        interest_paid = d.pop("interestPaid", UNSET)

        cash_equivalent = d.pop("cashEquivalent", UNSET)

        full_tax_months = d.pop("fullTaxMonths", UNSET)

        monthly_value = d.pop("monthlyValue", UNSET)

        official_interest = d.pop("officialInterest", UNSET)

        benefit_details_loan = cls(
            number_of_joint_borrowers=number_of_joint_borrowers,
            loan_made=loan_made,
            loan_discharged=loan_discharged,
            starting_balance=starting_balance,
            closing_balance=closing_balance,
            max_balance_in_year=max_balance_in_year,
            interest_paid=interest_paid,
            cash_equivalent=cash_equivalent,
            full_tax_months=full_tax_months,
            monthly_value=monthly_value,
            official_interest=official_interest,
        )

        return benefit_details_loan



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FieldModification")


@attr.s(auto_attribs=True)
class FieldModification:
    """
    Attributes:
        field_name (Union[Unset, None, str]):
        is_significant (Union[Unset, bool]):
        old_value (Union[Unset, None, str]):
        new_value (Union[Unset, None, str]):
    """

    field_name: Union[Unset, None, str] = UNSET
    is_significant: Union[Unset, bool] = UNSET
    old_value: Union[Unset, None, str] = UNSET
    new_value: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        field_name = self.field_name
        is_significant = self.is_significant
        old_value = self.old_value
        new_value = self.new_value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if field_name is not UNSET:
            field_dict["fieldName"] = field_name
        if is_significant is not UNSET:
            field_dict["isSignificant"] = is_significant
        if old_value is not UNSET:
            field_dict["oldValue"] = old_value
        if new_value is not UNSET:
            field_dict["newValue"] = new_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field_name = d.pop("fieldName", UNSET)

        is_significant = d.pop("isSignificant", UNSET)

        old_value = d.pop("oldValue", UNSET)

        new_value = d.pop("newValue", UNSET)

        field_modification = cls(
            field_name=field_name,
            is_significant=is_significant,
            old_value=old_value,
            new_value=new_value,
        )

        return field_modification



from setuptools import find_packages, setup

with open('README.md') as f:
    long_description = f.read()

setup(
    name='staffology',
    version='0.1.2',
    packages=find_packages(),
    url='https://github.com/ihorizonUK/staffology',
    description="Staffology openapi client with authentication patch",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='isik-kaplan',
    author_email='isik@ihorizon.co.uk',
    python_requires=">=3.7",
    install_requires=['httpx', "attrs"],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Development Status :: 5 - Production/Stable',
    ],
)



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.report_response import ReportResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/billing/usage-stats".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(accept, Unset):
        headers["accept"] = accept

    params: Dict[str, Any] = {}
    json_from_date: Union[Unset, None, str] = UNSET
    if not isinstance(from_date, Unset):
        json_from_date = from_date.isoformat() if from_date else None

    params["fromDate"] = json_from_date

    json_to_date: Union[Unset, None, str] = UNSET
    if not isinstance(to_date, Unset):
        json_to_date = to_date.isoformat() if to_date else None

    params["toDate"] = json_to_date

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ReportResponse]:
    if response.status_code == 200:
        response_200 = ReportResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ReportResponse]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Get UsageStats Report

     Returns usage statistics data for all employers the logged in user has access to for the given date
    range.
    If either of the dates are not provided then the values are defaulted to the first and last date of
    the previous calendar month.

    Args:
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        from_date=from_date,
        to_date=to_date,
        accept=accept,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Get UsageStats Report

     Returns usage statistics data for all employers the logged in user has access to for the given date
    range.
    If either of the dates are not provided then the values are defaulted to the first and last date of
    the previous calendar month.

    Args:
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return sync_detailed(
        client=client,
        from_date=from_date,
        to_date=to_date,
        accept=accept,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Get UsageStats Report

     Returns usage statistics data for all employers the logged in user has access to for the given date
    range.
    If either of the dates are not provided then the values are defaulted to the first and last date of
    the previous calendar month.

    Args:
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        from_date=from_date,
        to_date=to_date,
        accept=accept,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Get UsageStats Report

     Returns usage statistics data for all employers the logged in user has access to for the given date
    range.
    If either of the dates are not provided then the values are defaulted to the first and last date of
    the previous calendar month.

    Args:
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            from_date=from_date,
            to_date=to_date,
            accept=accept,
        )
    ).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.tenant import Tenant
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: Tenant,
) -> Dict[str, Any]:
    url = "{}/tenants".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, Tenant]]:
    if response.status_code == 201:
        response_201 = Tenant.from_dict(response.json())

        return response_201
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, Tenant]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: Tenant,
) -> Response[Union[Any, Tenant]]:
    """Create a Tenant

     Use this method to create a new Tenant
    Only available to SuperAdmins

    Args:
        json_body (Tenant): The Tenant model represents the brand that provides the account.
            This is used by our White Label partners to manage and brand their user accounts.
            Unless you are an admin for a White Label account you'll have no interest in this model.

    Returns:
        Response[Union[Any, Tenant]]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    json_body: Tenant,
) -> Optional[Union[Any, Tenant]]:
    """Create a Tenant

     Use this method to create a new Tenant
    Only available to SuperAdmins

    Args:
        json_body (Tenant): The Tenant model represents the brand that provides the account.
            This is used by our White Label partners to manage and brand their user accounts.
            Unless you are an admin for a White Label account you'll have no interest in this model.

    Returns:
        Response[Union[Any, Tenant]]
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    json_body: Tenant,
) -> Response[Union[Any, Tenant]]:
    """Create a Tenant

     Use this method to create a new Tenant
    Only available to SuperAdmins

    Args:
        json_body (Tenant): The Tenant model represents the brand that provides the account.
            This is used by our White Label partners to manage and brand their user accounts.
            Unless you are an admin for a White Label account you'll have no interest in this model.

    Returns:
        Response[Union[Any, Tenant]]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    json_body: Tenant,
) -> Optional[Union[Any, Tenant]]:
    """Create a Tenant

     Use this method to create a new Tenant
    Only available to SuperAdmins

    Args:
        json_body (Tenant): The Tenant model represents the brand that provides the account.
            This is used by our White Label partners to manage and brand their user accounts.
            Unless you are an admin for a White Label account you'll have no interest in this model.

    Returns:
        Response[Union[Any, Tenant]]
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NationalMinimumWage")


@attr.s(auto_attribs=True)
class NationalMinimumWage:
    """Part of the TaxYearConfig that our engine uses to calculate National Minimum Wage.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            apprentice (Union[Unset, None, bool]):
            max_age (Union[Unset, int]):
            hourly_amount (Union[Unset, float]):
    """

    apprentice: Union[Unset, None, bool] = UNSET
    max_age: Union[Unset, int] = UNSET
    hourly_amount: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        apprentice = self.apprentice
        max_age = self.max_age
        hourly_amount = self.hourly_amount

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if apprentice is not UNSET:
            field_dict["apprentice"] = apprentice
        if max_age is not UNSET:
            field_dict["maxAge"] = max_age
        if hourly_amount is not UNSET:
            field_dict["hourlyAmount"] = hourly_amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        apprentice = d.pop("apprentice", UNSET)

        max_age = d.pop("maxAge", UNSET)

        hourly_amount = d.pop("hourlyAmount", UNSET)

        national_minimum_wage = cls(
            apprentice=apprentice,
            max_age=max_age,
            hourly_amount=hourly_amount,
        )

        return national_minimum_wage



from enum import Enum


class PensionContributionLevelType(str, Enum):
    USERDEFINED = "UserDefined"
    STATUTORYMINIMUM = "StatutoryMinimum"
    NHS2015 = "Nhs2015"
    TP2020 = "Tp2020"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.hmrc_liability import HmrcLiability
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="P32")


@attr.s(auto_attribs=True)
class P32:
    """
    Attributes:
        is_quarterly (Union[Unset, bool]): [readonly]
        hmrc_liabilities (Union[Unset, None, List[HmrcLiability]]): [readonly]
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    is_quarterly: Union[Unset, bool] = UNSET
    hmrc_liabilities: Union[Unset, None, List[HmrcLiability]] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_quarterly = self.is_quarterly
        hmrc_liabilities: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.hmrc_liabilities, Unset):
            if self.hmrc_liabilities is None:
                hmrc_liabilities = None
            else:
                hmrc_liabilities = []
                for hmrc_liabilities_item_data in self.hmrc_liabilities:
                    hmrc_liabilities_item = hmrc_liabilities_item_data.to_dict()

                    hmrc_liabilities.append(hmrc_liabilities_item)

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_quarterly is not UNSET:
            field_dict["isQuarterly"] = is_quarterly
        if hmrc_liabilities is not UNSET:
            field_dict["hmrcLiabilities"] = hmrc_liabilities
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_quarterly = d.pop("isQuarterly", UNSET)

        hmrc_liabilities = []
        _hmrc_liabilities = d.pop("hmrcLiabilities", UNSET)
        for hmrc_liabilities_item_data in _hmrc_liabilities or []:
            hmrc_liabilities_item = HmrcLiability.from_dict(hmrc_liabilities_item_data)

            hmrc_liabilities.append(hmrc_liabilities_item)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        p32 = cls(
            is_quarterly=is_quarterly,
            hmrc_liabilities=hmrc_liabilities,
            report=report,
            tax_year=tax_year,
        )

        return p32



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.fps_car_fuel import FpsCarFuel
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsCar")


@attr.s(auto_attribs=True)
class FpsCar:
    """
    Attributes:
        make (Union[Unset, None, str]):
        first_regd (Union[Unset, None, str]):
        c_o2 (Union[Unset, None, str]):
        zero_emissions_mileage (Union[Unset, None, str]):
        fuel (Union[Unset, None, str]):
        id (Union[Unset, None, str]):
        amendment (Union[Unset, None, str]):
        price (Union[Unset, None, str]):
        avail_from (Union[Unset, None, str]):
        cash_equiv (Union[Unset, None, str]):
        avail_to (Union[Unset, None, str]):
        free_fuel (Union[Unset, FpsCarFuel]):
    """

    make: Union[Unset, None, str] = UNSET
    first_regd: Union[Unset, None, str] = UNSET
    c_o2: Union[Unset, None, str] = UNSET
    zero_emissions_mileage: Union[Unset, None, str] = UNSET
    fuel: Union[Unset, None, str] = UNSET
    id: Union[Unset, None, str] = UNSET
    amendment: Union[Unset, None, str] = UNSET
    price: Union[Unset, None, str] = UNSET
    avail_from: Union[Unset, None, str] = UNSET
    cash_equiv: Union[Unset, None, str] = UNSET
    avail_to: Union[Unset, None, str] = UNSET
    free_fuel: Union[Unset, FpsCarFuel] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        make = self.make
        first_regd = self.first_regd
        c_o2 = self.c_o2
        zero_emissions_mileage = self.zero_emissions_mileage
        fuel = self.fuel
        id = self.id
        amendment = self.amendment
        price = self.price
        avail_from = self.avail_from
        cash_equiv = self.cash_equiv
        avail_to = self.avail_to
        free_fuel: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.free_fuel, Unset):
            free_fuel = self.free_fuel.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if make is not UNSET:
            field_dict["make"] = make
        if first_regd is not UNSET:
            field_dict["firstRegd"] = first_regd
        if c_o2 is not UNSET:
            field_dict["cO2"] = c_o2
        if zero_emissions_mileage is not UNSET:
            field_dict["zeroEmissionsMileage"] = zero_emissions_mileage
        if fuel is not UNSET:
            field_dict["fuel"] = fuel
        if id is not UNSET:
            field_dict["id"] = id
        if amendment is not UNSET:
            field_dict["amendment"] = amendment
        if price is not UNSET:
            field_dict["price"] = price
        if avail_from is not UNSET:
            field_dict["availFrom"] = avail_from
        if cash_equiv is not UNSET:
            field_dict["cashEquiv"] = cash_equiv
        if avail_to is not UNSET:
            field_dict["availTo"] = avail_to
        if free_fuel is not UNSET:
            field_dict["freeFuel"] = free_fuel

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        make = d.pop("make", UNSET)

        first_regd = d.pop("firstRegd", UNSET)

        c_o2 = d.pop("cO2", UNSET)

        zero_emissions_mileage = d.pop("zeroEmissionsMileage", UNSET)

        fuel = d.pop("fuel", UNSET)

        id = d.pop("id", UNSET)

        amendment = d.pop("amendment", UNSET)

        price = d.pop("price", UNSET)

        avail_from = d.pop("availFrom", UNSET)

        cash_equiv = d.pop("cashEquiv", UNSET)

        avail_to = d.pop("availTo", UNSET)

        _free_fuel = d.pop("freeFuel", UNSET)
        free_fuel: Union[Unset, FpsCarFuel]
        if isinstance(_free_fuel, Unset):
            free_fuel = UNSET
        else:
            free_fuel = FpsCarFuel.from_dict(_free_fuel)

        fps_car = cls(
            make=make,
            first_regd=first_regd,
            c_o2=c_o2,
            zero_emissions_mileage=zero_emissions_mileage,
            fuel=fuel,
            id=id,
            amendment=amendment,
            price=price,
            avail_from=avail_from,
            cash_equiv=cash_equiv,
            avail_to=avail_to,
            free_fuel=free_fuel,
        )

        return fps_car



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.student_loan import StudentLoan
from ..types import UNSET, Unset

T = TypeVar("T", bound="TaxAndNi")


@attr.s(auto_attribs=True)
class TaxAndNi:
    """
    Attributes:
        ni_table (str): The appropriate NI letter for this Employee
        secondary_class_1_not_payable (Union[Unset, bool]): If set to true then no Employer NI will be paid for this
            Employee
        postgrad_loan (Union[Unset, bool]): Set to true if the Employee needs to make Post Graduate Loan repayments
        postgraduate_loan_start_date (Union[Unset, None, datetime.date]):
        postgraduate_loan_end_date (Union[Unset, None, datetime.date]):
        student_loan (Union[Unset, StudentLoan]):
        student_loan_start_date (Union[Unset, None, datetime.date]):
        student_loan_end_date (Union[Unset, None, datetime.date]):
        tax_code (Union[Unset, None, str]): The Tax Code for this Employee
        week_1_month_1 (Union[Unset, bool]): Determines whether PAYE should be calculated on a Week1/Month1 basis
            instead of on a cumulative basis.
            This is automatically set to false for any existing Employees when you start a new Tax Year.
    """

    ni_table: str
    secondary_class_1_not_payable: Union[Unset, bool] = UNSET
    postgrad_loan: Union[Unset, bool] = UNSET
    postgraduate_loan_start_date: Union[Unset, None, datetime.date] = UNSET
    postgraduate_loan_end_date: Union[Unset, None, datetime.date] = UNSET
    student_loan: Union[Unset, StudentLoan] = UNSET
    student_loan_start_date: Union[Unset, None, datetime.date] = UNSET
    student_loan_end_date: Union[Unset, None, datetime.date] = UNSET
    tax_code: Union[Unset, None, str] = UNSET
    week_1_month_1: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ni_table = self.ni_table
        secondary_class_1_not_payable = self.secondary_class_1_not_payable
        postgrad_loan = self.postgrad_loan
        postgraduate_loan_start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.postgraduate_loan_start_date, Unset):
            postgraduate_loan_start_date = (
                self.postgraduate_loan_start_date.isoformat()
                if self.postgraduate_loan_start_date
                else None
            )

        postgraduate_loan_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.postgraduate_loan_end_date, Unset):
            postgraduate_loan_end_date = (
                self.postgraduate_loan_end_date.isoformat()
                if self.postgraduate_loan_end_date
                else None
            )

        student_loan: Union[Unset, str] = UNSET
        if not isinstance(self.student_loan, Unset):
            student_loan = self.student_loan.value

        student_loan_start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.student_loan_start_date, Unset):
            student_loan_start_date = (
                self.student_loan_start_date.isoformat()
                if self.student_loan_start_date
                else None
            )

        student_loan_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.student_loan_end_date, Unset):
            student_loan_end_date = (
                self.student_loan_end_date.isoformat()
                if self.student_loan_end_date
                else None
            )

        tax_code = self.tax_code
        week_1_month_1 = self.week_1_month_1

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "niTable": ni_table,
            }
        )
        if secondary_class_1_not_payable is not UNSET:
            field_dict["secondaryClass1NotPayable"] = secondary_class_1_not_payable
        if postgrad_loan is not UNSET:
            field_dict["postgradLoan"] = postgrad_loan
        if postgraduate_loan_start_date is not UNSET:
            field_dict["postgraduateLoanStartDate"] = postgraduate_loan_start_date
        if postgraduate_loan_end_date is not UNSET:
            field_dict["postgraduateLoanEndDate"] = postgraduate_loan_end_date
        if student_loan is not UNSET:
            field_dict["studentLoan"] = student_loan
        if student_loan_start_date is not UNSET:
            field_dict["studentLoanStartDate"] = student_loan_start_date
        if student_loan_end_date is not UNSET:
            field_dict["studentLoanEndDate"] = student_loan_end_date
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code
        if week_1_month_1 is not UNSET:
            field_dict["week1Month1"] = week_1_month_1

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ni_table = d.pop("niTable")

        secondary_class_1_not_payable = d.pop("secondaryClass1NotPayable", UNSET)

        postgrad_loan = d.pop("postgradLoan", UNSET)

        _postgraduate_loan_start_date = d.pop("postgraduateLoanStartDate", UNSET)
        postgraduate_loan_start_date: Union[Unset, None, datetime.date]
        if _postgraduate_loan_start_date is None:
            postgraduate_loan_start_date = None
        elif isinstance(_postgraduate_loan_start_date, Unset):
            postgraduate_loan_start_date = UNSET
        else:
            postgraduate_loan_start_date = isoparse(
                _postgraduate_loan_start_date
            ).date()

        _postgraduate_loan_end_date = d.pop("postgraduateLoanEndDate", UNSET)
        postgraduate_loan_end_date: Union[Unset, None, datetime.date]
        if _postgraduate_loan_end_date is None:
            postgraduate_loan_end_date = None
        elif isinstance(_postgraduate_loan_end_date, Unset):
            postgraduate_loan_end_date = UNSET
        else:
            postgraduate_loan_end_date = isoparse(_postgraduate_loan_end_date).date()

        _student_loan = d.pop("studentLoan", UNSET)
        student_loan: Union[Unset, StudentLoan]
        if isinstance(_student_loan, Unset):
            student_loan = UNSET
        else:
            student_loan = StudentLoan(_student_loan)

        _student_loan_start_date = d.pop("studentLoanStartDate", UNSET)
        student_loan_start_date: Union[Unset, None, datetime.date]
        if _student_loan_start_date is None:
            student_loan_start_date = None
        elif isinstance(_student_loan_start_date, Unset):
            student_loan_start_date = UNSET
        else:
            student_loan_start_date = isoparse(_student_loan_start_date).date()

        _student_loan_end_date = d.pop("studentLoanEndDate", UNSET)
        student_loan_end_date: Union[Unset, None, datetime.date]
        if _student_loan_end_date is None:
            student_loan_end_date = None
        elif isinstance(_student_loan_end_date, Unset):
            student_loan_end_date = UNSET
        else:
            student_loan_end_date = isoparse(_student_loan_end_date).date()

        tax_code = d.pop("taxCode", UNSET)

        week_1_month_1 = d.pop("week1Month1", UNSET)

        tax_and_ni = cls(
            ni_table=ni_table,
            secondary_class_1_not_payable=secondary_class_1_not_payable,
            postgrad_loan=postgrad_loan,
            postgraduate_loan_start_date=postgraduate_loan_start_date,
            postgraduate_loan_end_date=postgraduate_loan_end_date,
            student_loan=student_loan,
            student_loan_start_date=student_loan_start_date,
            student_loan_end_date=student_loan_end_date,
            tax_code=tax_code,
            week_1_month_1=week_1_month_1,
        )

        return tax_and_ni



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..models.auto_enrolment_settings import AutoEnrolmentSettings
from ..models.bank_details import BankDetails
from ..models.bank_payment_instructions_csv_format import (
    BankPaymentInstructionsCsvFormat,
)
from ..models.employer_settings import EmployerSettings
from ..models.hmrc_details import HmrcDetails
from ..models.leave_settings import LeaveSettings
from ..models.pay_options import PayOptions
from ..models.pension_selection import PensionSelection
from ..models.rti_submission_settings import RtiSubmissionSettings
from ..models.tax_year import TaxYear
from ..models.umbrella_settings import UmbrellaSettings
from ..types import UNSET, Unset

T = TypeVar("T", bound="Employer")


@attr.s(auto_attribs=True)
class Employer:
    """
    Attributes:
        name (str):
        crn (Union[Unset, None, str]): Company Registration Number
        logo_url (Union[Unset, None, str]):
        alternative_id (Union[Unset, None, str]):
        bank_payments_csv_format (Union[Unset, BankPaymentInstructionsCsvFormat]):
        bacs_service_user_number (Union[Unset, None, str]):
        bacs_bureau_number (Union[Unset, None, str]):
        reject_invalid_bank_details (Union[Unset, bool]):
        bank_payments_reference_format (Union[Unset, None, str]):
        use_tenant_rti_submission_settings (Union[Unset, bool]): If the Tenant for this employer has Bureau Features
            enabled then they can set RtiSubmissionSettings to be used across multiple employers.
            If this is set to true then those settings will be used instead of any set at the Employer level
        address (Union[Unset, Address]):
        bank_details (Union[Unset, BankDetails]):
        default_pay_options (Union[Unset, PayOptions]): This object forms the basis of the Employees payment.
        hmrc_details (Union[Unset, HmrcDetails]):
        default_pension (Union[Unset, PensionSelection]):
        rti_submission_settings (Union[Unset, RtiSubmissionSettings]):
        auto_enrolment_settings (Union[Unset, AutoEnrolmentSettings]):
        leave_settings (Union[Unset, LeaveSettings]):
        settings (Union[Unset, EmployerSettings]): Miscellaneous settings related to the employer that don't naturally
            belong in other models
        umbrella_settings (Union[Unset, UmbrellaSettings]):
        employee_count (Union[Unset, int]): [readonly] The number of Employees this Employer has, including CIS
            Subcontractors.
        subcontractor_count (Union[Unset, int]): [readonly] The number of CIS Subcontractors this Employer has.
        start_year (Union[Unset, TaxYear]):
        current_year (Union[Unset, TaxYear]):
        support_access_enabled (Union[Unset, bool]): If set to true then the support team can access this employer to
            help resolve
            support queries
        archived (Union[Unset, bool]): A flag to indicate whather or not the employer is Archived, ie no longer actively
            used
        can_use_bureau_features (Union[Unset, bool]):
        id (Union[Unset, str]): [readonly] The unique id of the object
        source_system_id (Union[Unset, None, str]): [readonly] Can only be given a value when the employer is created.
            It can then never be changed.
            Used by external systems so they can store an immutable reference
    """

    name: str
    crn: Union[Unset, None, str] = UNSET
    logo_url: Union[Unset, None, str] = UNSET
    alternative_id: Union[Unset, None, str] = UNSET
    bank_payments_csv_format: Union[Unset, BankPaymentInstructionsCsvFormat] = UNSET
    bacs_service_user_number: Union[Unset, None, str] = UNSET
    bacs_bureau_number: Union[Unset, None, str] = UNSET
    reject_invalid_bank_details: Union[Unset, bool] = UNSET
    bank_payments_reference_format: Union[Unset, None, str] = UNSET
    use_tenant_rti_submission_settings: Union[Unset, bool] = UNSET
    address: Union[Unset, Address] = UNSET
    bank_details: Union[Unset, BankDetails] = UNSET
    default_pay_options: Union[Unset, PayOptions] = UNSET
    hmrc_details: Union[Unset, HmrcDetails] = UNSET
    default_pension: Union[Unset, PensionSelection] = UNSET
    rti_submission_settings: Union[Unset, RtiSubmissionSettings] = UNSET
    auto_enrolment_settings: Union[Unset, AutoEnrolmentSettings] = UNSET
    leave_settings: Union[Unset, LeaveSettings] = UNSET
    settings: Union[Unset, EmployerSettings] = UNSET
    umbrella_settings: Union[Unset, UmbrellaSettings] = UNSET
    employee_count: Union[Unset, int] = UNSET
    subcontractor_count: Union[Unset, int] = UNSET
    start_year: Union[Unset, TaxYear] = UNSET
    current_year: Union[Unset, TaxYear] = UNSET
    support_access_enabled: Union[Unset, bool] = UNSET
    archived: Union[Unset, bool] = UNSET
    can_use_bureau_features: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    source_system_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        crn = self.crn
        logo_url = self.logo_url
        alternative_id = self.alternative_id
        bank_payments_csv_format: Union[Unset, str] = UNSET
        if not isinstance(self.bank_payments_csv_format, Unset):
            bank_payments_csv_format = self.bank_payments_csv_format.value

        bacs_service_user_number = self.bacs_service_user_number
        bacs_bureau_number = self.bacs_bureau_number
        reject_invalid_bank_details = self.reject_invalid_bank_details
        bank_payments_reference_format = self.bank_payments_reference_format
        use_tenant_rti_submission_settings = self.use_tenant_rti_submission_settings
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        default_pay_options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_pay_options, Unset):
            default_pay_options = self.default_pay_options.to_dict()

        hmrc_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hmrc_details, Unset):
            hmrc_details = self.hmrc_details.to_dict()

        default_pension: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_pension, Unset):
            default_pension = self.default_pension.to_dict()

        rti_submission_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rti_submission_settings, Unset):
            rti_submission_settings = self.rti_submission_settings.to_dict()

        auto_enrolment_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_enrolment_settings, Unset):
            auto_enrolment_settings = self.auto_enrolment_settings.to_dict()

        leave_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.leave_settings, Unset):
            leave_settings = self.leave_settings.to_dict()

        settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        umbrella_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.umbrella_settings, Unset):
            umbrella_settings = self.umbrella_settings.to_dict()

        employee_count = self.employee_count
        subcontractor_count = self.subcontractor_count
        start_year: Union[Unset, str] = UNSET
        if not isinstance(self.start_year, Unset):
            start_year = self.start_year.value

        current_year: Union[Unset, str] = UNSET
        if not isinstance(self.current_year, Unset):
            current_year = self.current_year.value

        support_access_enabled = self.support_access_enabled
        archived = self.archived
        can_use_bureau_features = self.can_use_bureau_features
        id = self.id
        source_system_id = self.source_system_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if crn is not UNSET:
            field_dict["crn"] = crn
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url
        if alternative_id is not UNSET:
            field_dict["alternativeId"] = alternative_id
        if bank_payments_csv_format is not UNSET:
            field_dict["bankPaymentsCsvFormat"] = bank_payments_csv_format
        if bacs_service_user_number is not UNSET:
            field_dict["bacsServiceUserNumber"] = bacs_service_user_number
        if bacs_bureau_number is not UNSET:
            field_dict["bacsBureauNumber"] = bacs_bureau_number
        if reject_invalid_bank_details is not UNSET:
            field_dict["rejectInvalidBankDetails"] = reject_invalid_bank_details
        if bank_payments_reference_format is not UNSET:
            field_dict["bankPaymentsReferenceFormat"] = bank_payments_reference_format
        if use_tenant_rti_submission_settings is not UNSET:
            field_dict[
                "useTenantRtiSubmissionSettings"
            ] = use_tenant_rti_submission_settings
        if address is not UNSET:
            field_dict["address"] = address
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if default_pay_options is not UNSET:
            field_dict["defaultPayOptions"] = default_pay_options
        if hmrc_details is not UNSET:
            field_dict["hmrcDetails"] = hmrc_details
        if default_pension is not UNSET:
            field_dict["defaultPension"] = default_pension
        if rti_submission_settings is not UNSET:
            field_dict["rtiSubmissionSettings"] = rti_submission_settings
        if auto_enrolment_settings is not UNSET:
            field_dict["autoEnrolmentSettings"] = auto_enrolment_settings
        if leave_settings is not UNSET:
            field_dict["leaveSettings"] = leave_settings
        if settings is not UNSET:
            field_dict["settings"] = settings
        if umbrella_settings is not UNSET:
            field_dict["umbrellaSettings"] = umbrella_settings
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if subcontractor_count is not UNSET:
            field_dict["subcontractorCount"] = subcontractor_count
        if start_year is not UNSET:
            field_dict["startYear"] = start_year
        if current_year is not UNSET:
            field_dict["currentYear"] = current_year
        if support_access_enabled is not UNSET:
            field_dict["supportAccessEnabled"] = support_access_enabled
        if archived is not UNSET:
            field_dict["archived"] = archived
        if can_use_bureau_features is not UNSET:
            field_dict["canUseBureauFeatures"] = can_use_bureau_features
        if id is not UNSET:
            field_dict["id"] = id
        if source_system_id is not UNSET:
            field_dict["sourceSystemId"] = source_system_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        crn = d.pop("crn", UNSET)

        logo_url = d.pop("logoUrl", UNSET)

        alternative_id = d.pop("alternativeId", UNSET)

        _bank_payments_csv_format = d.pop("bankPaymentsCsvFormat", UNSET)
        bank_payments_csv_format: Union[Unset, BankPaymentInstructionsCsvFormat]
        if isinstance(_bank_payments_csv_format, Unset):
            bank_payments_csv_format = UNSET
        else:
            bank_payments_csv_format = BankPaymentInstructionsCsvFormat(
                _bank_payments_csv_format
            )

        bacs_service_user_number = d.pop("bacsServiceUserNumber", UNSET)

        bacs_bureau_number = d.pop("bacsBureauNumber", UNSET)

        reject_invalid_bank_details = d.pop("rejectInvalidBankDetails", UNSET)

        bank_payments_reference_format = d.pop("bankPaymentsReferenceFormat", UNSET)

        use_tenant_rti_submission_settings = d.pop(
            "useTenantRtiSubmissionSettings", UNSET
        )

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        _default_pay_options = d.pop("defaultPayOptions", UNSET)
        default_pay_options: Union[Unset, PayOptions]
        if isinstance(_default_pay_options, Unset):
            default_pay_options = UNSET
        else:
            default_pay_options = PayOptions.from_dict(_default_pay_options)

        _hmrc_details = d.pop("hmrcDetails", UNSET)
        hmrc_details: Union[Unset, HmrcDetails]
        if isinstance(_hmrc_details, Unset):
            hmrc_details = UNSET
        else:
            hmrc_details = HmrcDetails.from_dict(_hmrc_details)

        _default_pension = d.pop("defaultPension", UNSET)
        default_pension: Union[Unset, PensionSelection]
        if isinstance(_default_pension, Unset):
            default_pension = UNSET
        else:
            default_pension = PensionSelection.from_dict(_default_pension)

        _rti_submission_settings = d.pop("rtiSubmissionSettings", UNSET)
        rti_submission_settings: Union[Unset, RtiSubmissionSettings]
        if isinstance(_rti_submission_settings, Unset):
            rti_submission_settings = UNSET
        else:
            rti_submission_settings = RtiSubmissionSettings.from_dict(
                _rti_submission_settings
            )

        _auto_enrolment_settings = d.pop("autoEnrolmentSettings", UNSET)
        auto_enrolment_settings: Union[Unset, AutoEnrolmentSettings]
        if isinstance(_auto_enrolment_settings, Unset):
            auto_enrolment_settings = UNSET
        else:
            auto_enrolment_settings = AutoEnrolmentSettings.from_dict(
                _auto_enrolment_settings
            )

        _leave_settings = d.pop("leaveSettings", UNSET)
        leave_settings: Union[Unset, LeaveSettings]
        if isinstance(_leave_settings, Unset):
            leave_settings = UNSET
        else:
            leave_settings = LeaveSettings.from_dict(_leave_settings)

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, EmployerSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = EmployerSettings.from_dict(_settings)

        _umbrella_settings = d.pop("umbrellaSettings", UNSET)
        umbrella_settings: Union[Unset, UmbrellaSettings]
        if isinstance(_umbrella_settings, Unset):
            umbrella_settings = UNSET
        else:
            umbrella_settings = UmbrellaSettings.from_dict(_umbrella_settings)

        employee_count = d.pop("employeeCount", UNSET)

        subcontractor_count = d.pop("subcontractorCount", UNSET)

        _start_year = d.pop("startYear", UNSET)
        start_year: Union[Unset, TaxYear]
        if isinstance(_start_year, Unset):
            start_year = UNSET
        else:
            start_year = TaxYear(_start_year)

        _current_year = d.pop("currentYear", UNSET)
        current_year: Union[Unset, TaxYear]
        if isinstance(_current_year, Unset):
            current_year = UNSET
        else:
            current_year = TaxYear(_current_year)

        support_access_enabled = d.pop("supportAccessEnabled", UNSET)

        archived = d.pop("archived", UNSET)

        can_use_bureau_features = d.pop("canUseBureauFeatures", UNSET)

        id = d.pop("id", UNSET)

        source_system_id = d.pop("sourceSystemId", UNSET)

        employer = cls(
            name=name,
            crn=crn,
            logo_url=logo_url,
            alternative_id=alternative_id,
            bank_payments_csv_format=bank_payments_csv_format,
            bacs_service_user_number=bacs_service_user_number,
            bacs_bureau_number=bacs_bureau_number,
            reject_invalid_bank_details=reject_invalid_bank_details,
            bank_payments_reference_format=bank_payments_reference_format,
            use_tenant_rti_submission_settings=use_tenant_rti_submission_settings,
            address=address,
            bank_details=bank_details,
            default_pay_options=default_pay_options,
            hmrc_details=hmrc_details,
            default_pension=default_pension,
            rti_submission_settings=rti_submission_settings,
            auto_enrolment_settings=auto_enrolment_settings,
            leave_settings=leave_settings,
            settings=settings,
            umbrella_settings=umbrella_settings,
            employee_count=employee_count,
            subcontractor_count=subcontractor_count,
            start_year=start_year,
            current_year=current_year,
            support_access_enabled=support_access_enabled,
            archived=archived,
            can_use_bureau_features=can_use_bureau_features,
            id=id,
            source_system_id=source_system_id,
        )

        return employer



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.webhook_event import WebhookEvent
from ..models.webhook_payload import WebhookPayload
from ..types import UNSET, Unset

T = TypeVar("T", bound="Webhook")


@attr.s(auto_attribs=True)
class Webhook:
    """
    Attributes:
        url (str): The Url to which the payload should be sent
        webhook_event (Union[Unset, WebhookEvent]):
        active (Union[Unset, bool]): If set to false then this Webhook will not be triggered
        event_count (Union[Unset, int]): The number of times this webhook has been triggered
        last_payload (Union[Unset, WebhookPayload]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    url: str
    webhook_event: Union[Unset, WebhookEvent] = UNSET
    active: Union[Unset, bool] = UNSET
    event_count: Union[Unset, int] = UNSET
    last_payload: Union[Unset, WebhookPayload] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        webhook_event: Union[Unset, str] = UNSET
        if not isinstance(self.webhook_event, Unset):
            webhook_event = self.webhook_event.value

        active = self.active
        event_count = self.event_count
        last_payload: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.last_payload, Unset):
            last_payload = self.last_payload.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "url": url,
            }
        )
        if webhook_event is not UNSET:
            field_dict["webhookEvent"] = webhook_event
        if active is not UNSET:
            field_dict["active"] = active
        if event_count is not UNSET:
            field_dict["eventCount"] = event_count
        if last_payload is not UNSET:
            field_dict["lastPayload"] = last_payload
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        _webhook_event = d.pop("webhookEvent", UNSET)
        webhook_event: Union[Unset, WebhookEvent]
        if isinstance(_webhook_event, Unset):
            webhook_event = UNSET
        else:
            webhook_event = WebhookEvent(_webhook_event)

        active = d.pop("active", UNSET)

        event_count = d.pop("eventCount", UNSET)

        _last_payload = d.pop("lastPayload", UNSET)
        last_payload: Union[Unset, WebhookPayload]
        if isinstance(_last_payload, Unset):
            last_payload = UNSET
        else:
            last_payload = WebhookPayload.from_dict(_last_payload)

        id = d.pop("id", UNSET)

        webhook = cls(
            url=url,
            webhook_event=webhook_event,
            active=active,
            event_count=event_count,
            last_payload=last_payload,
            id=id,
        )

        return webhook



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cis_partnership import CisPartnership
from ..models.item import Item
from ..models.rti_employee_address import RtiEmployeeAddress
from ..models.rti_employee_name import RtiEmployeeName
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisSubContractor")


@attr.s(auto_attribs=True)
class CisSubContractor:
    """
    Attributes:
        employee_unique_id (Union[Unset, str]):
        email_statement_to (Union[Unset, None, str]):
        number_of_payments (Union[Unset, int]):
        item (Union[Unset, Item]):
        display_name (Union[Unset, None, str]):
        action (Union[Unset, None, str]):
        type (Union[Unset, None, str]):
        name (Union[Unset, RtiEmployeeName]):
        trading_name (Union[Unset, None, str]):
        works_ref (Union[Unset, None, str]):
        unmatched_rate (Union[Unset, None, str]):
        utr (Union[Unset, None, str]):
        crn (Union[Unset, None, str]):
        nino (Union[Unset, None, str]):
        partnership (Union[Unset, CisPartnership]): If an Employee is marked as a CIS Subcontractor and is registered as
            a Partnership then this model provides further details specifically related to the CIS Partnership.
        address (Union[Unset, RtiEmployeeAddress]):
        telephone (Union[Unset, None, str]):
        total_payments_unrounded (Union[Unset, None, str]):
        cost_of_materials_unrounded (Union[Unset, None, str]):
        umbrella_fee (Union[Unset, None, str]):
        validation_msg (Union[Unset, None, str]):
        verification_number (Union[Unset, None, str]):
        total_payments (Union[Unset, None, str]):
        cost_of_materials (Union[Unset, None, str]):
        total_deducted (Union[Unset, None, str]):
        matched (Union[Unset, None, str]):
        tax_treatment (Union[Unset, None, str]):
    """

    employee_unique_id: Union[Unset, str] = UNSET
    email_statement_to: Union[Unset, None, str] = UNSET
    number_of_payments: Union[Unset, int] = UNSET
    item: Union[Unset, Item] = UNSET
    display_name: Union[Unset, None, str] = UNSET
    action: Union[Unset, None, str] = UNSET
    type: Union[Unset, None, str] = UNSET
    name: Union[Unset, RtiEmployeeName] = UNSET
    trading_name: Union[Unset, None, str] = UNSET
    works_ref: Union[Unset, None, str] = UNSET
    unmatched_rate: Union[Unset, None, str] = UNSET
    utr: Union[Unset, None, str] = UNSET
    crn: Union[Unset, None, str] = UNSET
    nino: Union[Unset, None, str] = UNSET
    partnership: Union[Unset, CisPartnership] = UNSET
    address: Union[Unset, RtiEmployeeAddress] = UNSET
    telephone: Union[Unset, None, str] = UNSET
    total_payments_unrounded: Union[Unset, None, str] = UNSET
    cost_of_materials_unrounded: Union[Unset, None, str] = UNSET
    umbrella_fee: Union[Unset, None, str] = UNSET
    validation_msg: Union[Unset, None, str] = UNSET
    verification_number: Union[Unset, None, str] = UNSET
    total_payments: Union[Unset, None, str] = UNSET
    cost_of_materials: Union[Unset, None, str] = UNSET
    total_deducted: Union[Unset, None, str] = UNSET
    matched: Union[Unset, None, str] = UNSET
    tax_treatment: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_unique_id = self.employee_unique_id
        email_statement_to = self.email_statement_to
        number_of_payments = self.number_of_payments
        item: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.item, Unset):
            item = self.item.to_dict()

        display_name = self.display_name
        action = self.action
        type = self.type
        name: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.to_dict()

        trading_name = self.trading_name
        works_ref = self.works_ref
        unmatched_rate = self.unmatched_rate
        utr = self.utr
        crn = self.crn
        nino = self.nino
        partnership: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.partnership, Unset):
            partnership = self.partnership.to_dict()

        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        telephone = self.telephone
        total_payments_unrounded = self.total_payments_unrounded
        cost_of_materials_unrounded = self.cost_of_materials_unrounded
        umbrella_fee = self.umbrella_fee
        validation_msg = self.validation_msg
        verification_number = self.verification_number
        total_payments = self.total_payments
        cost_of_materials = self.cost_of_materials
        total_deducted = self.total_deducted
        matched = self.matched
        tax_treatment = self.tax_treatment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_unique_id is not UNSET:
            field_dict["employeeUniqueId"] = employee_unique_id
        if email_statement_to is not UNSET:
            field_dict["emailStatementTo"] = email_statement_to
        if number_of_payments is not UNSET:
            field_dict["numberOfPayments"] = number_of_payments
        if item is not UNSET:
            field_dict["item"] = item
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if action is not UNSET:
            field_dict["action"] = action
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if trading_name is not UNSET:
            field_dict["tradingName"] = trading_name
        if works_ref is not UNSET:
            field_dict["worksRef"] = works_ref
        if unmatched_rate is not UNSET:
            field_dict["unmatchedRate"] = unmatched_rate
        if utr is not UNSET:
            field_dict["utr"] = utr
        if crn is not UNSET:
            field_dict["crn"] = crn
        if nino is not UNSET:
            field_dict["nino"] = nino
        if partnership is not UNSET:
            field_dict["partnership"] = partnership
        if address is not UNSET:
            field_dict["address"] = address
        if telephone is not UNSET:
            field_dict["telephone"] = telephone
        if total_payments_unrounded is not UNSET:
            field_dict["totalPaymentsUnrounded"] = total_payments_unrounded
        if cost_of_materials_unrounded is not UNSET:
            field_dict["costOfMaterialsUnrounded"] = cost_of_materials_unrounded
        if umbrella_fee is not UNSET:
            field_dict["umbrellaFee"] = umbrella_fee
        if validation_msg is not UNSET:
            field_dict["validationMsg"] = validation_msg
        if verification_number is not UNSET:
            field_dict["verificationNumber"] = verification_number
        if total_payments is not UNSET:
            field_dict["totalPayments"] = total_payments
        if cost_of_materials is not UNSET:
            field_dict["costOfMaterials"] = cost_of_materials
        if total_deducted is not UNSET:
            field_dict["totalDeducted"] = total_deducted
        if matched is not UNSET:
            field_dict["matched"] = matched
        if tax_treatment is not UNSET:
            field_dict["taxTreatment"] = tax_treatment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_unique_id = d.pop("employeeUniqueId", UNSET)

        email_statement_to = d.pop("emailStatementTo", UNSET)

        number_of_payments = d.pop("numberOfPayments", UNSET)

        _item = d.pop("item", UNSET)
        item: Union[Unset, Item]
        if isinstance(_item, Unset):
            item = UNSET
        else:
            item = Item.from_dict(_item)

        display_name = d.pop("displayName", UNSET)

        action = d.pop("action", UNSET)

        type = d.pop("type", UNSET)

        _name = d.pop("name", UNSET)
        name: Union[Unset, RtiEmployeeName]
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = RtiEmployeeName.from_dict(_name)

        trading_name = d.pop("tradingName", UNSET)

        works_ref = d.pop("worksRef", UNSET)

        unmatched_rate = d.pop("unmatchedRate", UNSET)

        utr = d.pop("utr", UNSET)

        crn = d.pop("crn", UNSET)

        nino = d.pop("nino", UNSET)

        _partnership = d.pop("partnership", UNSET)
        partnership: Union[Unset, CisPartnership]
        if isinstance(_partnership, Unset):
            partnership = UNSET
        else:
            partnership = CisPartnership.from_dict(_partnership)

        _address = d.pop("address", UNSET)
        address: Union[Unset, RtiEmployeeAddress]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = RtiEmployeeAddress.from_dict(_address)

        telephone = d.pop("telephone", UNSET)

        total_payments_unrounded = d.pop("totalPaymentsUnrounded", UNSET)

        cost_of_materials_unrounded = d.pop("costOfMaterialsUnrounded", UNSET)

        umbrella_fee = d.pop("umbrellaFee", UNSET)

        validation_msg = d.pop("validationMsg", UNSET)

        verification_number = d.pop("verificationNumber", UNSET)

        total_payments = d.pop("totalPayments", UNSET)

        cost_of_materials = d.pop("costOfMaterials", UNSET)

        total_deducted = d.pop("totalDeducted", UNSET)

        matched = d.pop("matched", UNSET)

        tax_treatment = d.pop("taxTreatment", UNSET)

        cis_sub_contractor = cls(
            employee_unique_id=employee_unique_id,
            email_statement_to=email_statement_to,
            number_of_payments=number_of_payments,
            item=item,
            display_name=display_name,
            action=action,
            type=type,
            name=name,
            trading_name=trading_name,
            works_ref=works_ref,
            unmatched_rate=unmatched_rate,
            utr=utr,
            crn=crn,
            nino=nino,
            partnership=partnership,
            address=address,
            telephone=telephone,
            total_payments_unrounded=total_payments_unrounded,
            cost_of_materials_unrounded=cost_of_materials_unrounded,
            umbrella_fee=umbrella_fee,
            validation_msg=validation_msg,
            verification_number=verification_number,
            total_payments=total_payments,
            cost_of_materials=cost_of_materials,
            total_deducted=total_deducted,
            matched=matched,
            tax_treatment=tax_treatment,
        )

        return cis_sub_contractor



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.dps_data_type import DpsDataType
from ..models.item import Item
from ..models.student_loan import StudentLoan
from ..types import UNSET, Unset

T = TypeVar("T", bound="DpsNotice")


@attr.s(auto_attribs=True)
class DpsNotice:
    """
    Attributes:
        issue_date (Union[Unset, datetime.date]):
        effective_date (Union[Unset, datetime.date]):
        tax_year (Union[Unset, int]):
        sequence_number (Union[Unset, int]):
        type (Union[Unset, DpsDataType]):
        form_type (Union[Unset, None, str]):
        forename (Union[Unset, None, str]):
        surname (Union[Unset, None, str]):
        ni_no (Union[Unset, None, str]):
        works_number (Union[Unset, None, str]):
        tax_code (Union[Unset, None, str]):
        week_1_month_1 (Union[Unset, None, bool]):
        previous_tax (Union[Unset, None, float]):
        previous_pay (Union[Unset, None, float]):
        plan_type (Union[Unset, StudentLoan]):
        applied_on (Union[Unset, None, datetime.date]):
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    issue_date: Union[Unset, datetime.date] = UNSET
    effective_date: Union[Unset, datetime.date] = UNSET
    tax_year: Union[Unset, int] = UNSET
    sequence_number: Union[Unset, int] = UNSET
    type: Union[Unset, DpsDataType] = UNSET
    form_type: Union[Unset, None, str] = UNSET
    forename: Union[Unset, None, str] = UNSET
    surname: Union[Unset, None, str] = UNSET
    ni_no: Union[Unset, None, str] = UNSET
    works_number: Union[Unset, None, str] = UNSET
    tax_code: Union[Unset, None, str] = UNSET
    week_1_month_1: Union[Unset, None, bool] = UNSET
    previous_tax: Union[Unset, None, float] = UNSET
    previous_pay: Union[Unset, None, float] = UNSET
    plan_type: Union[Unset, StudentLoan] = UNSET
    applied_on: Union[Unset, None, datetime.date] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        issue_date: Union[Unset, str] = UNSET
        if not isinstance(self.issue_date, Unset):
            issue_date = self.issue_date.isoformat()

        effective_date: Union[Unset, str] = UNSET
        if not isinstance(self.effective_date, Unset):
            effective_date = self.effective_date.isoformat()

        tax_year = self.tax_year
        sequence_number = self.sequence_number
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        form_type = self.form_type
        forename = self.forename
        surname = self.surname
        ni_no = self.ni_no
        works_number = self.works_number
        tax_code = self.tax_code
        week_1_month_1 = self.week_1_month_1
        previous_tax = self.previous_tax
        previous_pay = self.previous_pay
        plan_type: Union[Unset, str] = UNSET
        if not isinstance(self.plan_type, Unset):
            plan_type = self.plan_type.value

        applied_on: Union[Unset, None, str] = UNSET
        if not isinstance(self.applied_on, Unset):
            applied_on = self.applied_on.isoformat() if self.applied_on else None

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if issue_date is not UNSET:
            field_dict["issueDate"] = issue_date
        if effective_date is not UNSET:
            field_dict["effectiveDate"] = effective_date
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if sequence_number is not UNSET:
            field_dict["sequenceNumber"] = sequence_number
        if type is not UNSET:
            field_dict["type"] = type
        if form_type is not UNSET:
            field_dict["formType"] = form_type
        if forename is not UNSET:
            field_dict["forename"] = forename
        if surname is not UNSET:
            field_dict["surname"] = surname
        if ni_no is not UNSET:
            field_dict["niNo"] = ni_no
        if works_number is not UNSET:
            field_dict["worksNumber"] = works_number
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code
        if week_1_month_1 is not UNSET:
            field_dict["week1Month1"] = week_1_month_1
        if previous_tax is not UNSET:
            field_dict["previousTax"] = previous_tax
        if previous_pay is not UNSET:
            field_dict["previousPay"] = previous_pay
        if plan_type is not UNSET:
            field_dict["planType"] = plan_type
        if applied_on is not UNSET:
            field_dict["appliedOn"] = applied_on
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _issue_date = d.pop("issueDate", UNSET)
        issue_date: Union[Unset, datetime.date]
        if isinstance(_issue_date, Unset):
            issue_date = UNSET
        else:
            issue_date = isoparse(_issue_date).date()

        _effective_date = d.pop("effectiveDate", UNSET)
        effective_date: Union[Unset, datetime.date]
        if isinstance(_effective_date, Unset):
            effective_date = UNSET
        else:
            effective_date = isoparse(_effective_date).date()

        tax_year = d.pop("taxYear", UNSET)

        sequence_number = d.pop("sequenceNumber", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, DpsDataType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DpsDataType(_type)

        form_type = d.pop("formType", UNSET)

        forename = d.pop("forename", UNSET)

        surname = d.pop("surname", UNSET)

        ni_no = d.pop("niNo", UNSET)

        works_number = d.pop("worksNumber", UNSET)

        tax_code = d.pop("taxCode", UNSET)

        week_1_month_1 = d.pop("week1Month1", UNSET)

        previous_tax = d.pop("previousTax", UNSET)

        previous_pay = d.pop("previousPay", UNSET)

        _plan_type = d.pop("planType", UNSET)
        plan_type: Union[Unset, StudentLoan]
        if isinstance(_plan_type, Unset):
            plan_type = UNSET
        else:
            plan_type = StudentLoan(_plan_type)

        _applied_on = d.pop("appliedOn", UNSET)
        applied_on: Union[Unset, None, datetime.date]
        if _applied_on is None:
            applied_on = None
        elif isinstance(_applied_on, Unset):
            applied_on = UNSET
        else:
            applied_on = isoparse(_applied_on).date()

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        dps_notice = cls(
            issue_date=issue_date,
            effective_date=effective_date,
            tax_year=tax_year,
            sequence_number=sequence_number,
            type=type,
            form_type=form_type,
            forename=forename,
            surname=surname,
            ni_no=ni_no,
            works_number=works_number,
            tax_code=tax_code,
            week_1_month_1=week_1_month_1,
            previous_tax=previous_tax,
            previous_pay=previous_pay,
            plan_type=plan_type,
            applied_on=applied_on,
            employee=employee,
            id=id,
        )

        return dps_notice



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.fps import Fps
from ...models.tax_year import TaxYear
from ...types import Response


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/rti/fps/{taxYear}/{id}/markasaccepted".format(
        client.base_url, employerId=employer_id, taxYear=tax_year, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Fps]:
    if response.status_code == 200:
        response_200 = Fps.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[Fps]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    id: str,
    *,
    client: Client,
) -> Response[Fps]:
    """Mark as Sent

     Marks an Full Payment Submission as having been sent to HMRC and accepted by them.
    You only use this method if the FPS has been submitted via an external system.
    It will automatically be updated as Sent and/or Accepted if it's submitted via this API.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        id (str):

    Returns:
        Response[Fps]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        id=id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    id: str,
    *,
    client: Client,
) -> Optional[Fps]:
    """Mark as Sent

     Marks an Full Payment Submission as having been sent to HMRC and accepted by them.
    You only use this method if the FPS has been submitted via an external system.
    It will automatically be updated as Sent and/or Accepted if it's submitted via this API.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        id (str):

    Returns:
        Response[Fps]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    id: str,
    *,
    client: Client,
) -> Response[Fps]:
    """Mark as Sent

     Marks an Full Payment Submission as having been sent to HMRC and accepted by them.
    You only use this method if the FPS has been submitted via an external system.
    It will automatically be updated as Sent and/or Accepted if it's submitted via this API.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        id (str):

    Returns:
        Response[Fps]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    id: str,
    *,
    client: Client,
) -> Optional[Fps]:
    """Mark as Sent

     Marks an Full Payment Submission as having been sent to HMRC and accepted by them.
    You only use this method if the FPS has been submitted via an external system.
    It will automatically be updated as Sent and/or Accepted if it's submitted via this API.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        id (str):

    Returns:
        Response[Fps]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            id=id,
            client=client,
        )
    ).parsed



from enum import Enum


class CsvFileFormat(str, Enum):
    MONEYSOFTEMPLOYEES = "MoneysoftEmployees"
    BRIGHTPAYEMPLOYEES = "BrightPayEmployees"
    FPS = "Fps"
    SAGEEMPLOYEEDETAILS = "SageEmployeeDetails"
    ACCESSPEOPLEPLANNER = "AccessPeoplePlanner"
    EMPLOYEECSV = "EmployeeCsv"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.cis_contractor import CisContractor
from ..models.cis_sub_contractor import CisSubContractor
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisRequest")


@attr.s(auto_attribs=True)
class CisRequest:
    """
    Attributes:
        contractor (Union[Unset, CisContractor]): Used to represent details of a CIS SubContractor when communicating
            with the HMRC Gateway
        subcontractor (Union[Unset, None, List[CisSubContractor]]):
        declaration (Union[Unset, None, str]):
    """

    contractor: Union[Unset, CisContractor] = UNSET
    subcontractor: Union[Unset, None, List[CisSubContractor]] = UNSET
    declaration: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        contractor: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contractor, Unset):
            contractor = self.contractor.to_dict()

        subcontractor: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.subcontractor, Unset):
            if self.subcontractor is None:
                subcontractor = None
            else:
                subcontractor = []
                for subcontractor_item_data in self.subcontractor:
                    subcontractor_item = subcontractor_item_data.to_dict()

                    subcontractor.append(subcontractor_item)

        declaration = self.declaration

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if contractor is not UNSET:
            field_dict["contractor"] = contractor
        if subcontractor is not UNSET:
            field_dict["subcontractor"] = subcontractor
        if declaration is not UNSET:
            field_dict["declaration"] = declaration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _contractor = d.pop("contractor", UNSET)
        contractor: Union[Unset, CisContractor]
        if isinstance(_contractor, Unset):
            contractor = UNSET
        else:
            contractor = CisContractor.from_dict(_contractor)

        subcontractor = []
        _subcontractor = d.pop("subcontractor", UNSET)
        for subcontractor_item_data in _subcontractor or []:
            subcontractor_item = CisSubContractor.from_dict(subcontractor_item_data)

            subcontractor.append(subcontractor_item)

        declaration = d.pop("declaration", UNSET)

        cis_request = cls(
            contractor=contractor,
            subcontractor=subcontractor,
            declaration=declaration,
        )

        return cis_request
